#include "pool.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

/* for ident(1) command */
static char id[] = "@(#) $Id: pool.c,v 1.3 2001/04/08 22:31:21 papillau Exp $";

int buffer_pool_init(struct pool *p, int nb, int size)
{
  int i;

  p->buf_list = (struct buffer *)malloc(sizeof(struct buffer)*nb);
  if (p->buf_list == NULL)
    return 0;

  p->buf_nb = nb;

  for (i=0;i<nb;i++)
    {
      p->buf_list[i].buf  = malloc(size);
      if (p->buf_list[i].buf == NULL)
	{
	  /* we should also free all the memory we allocated so far */
	  return 0;
	}
      p->buf_list[i].size = size;
      p->buf_list[i].allocated = 0;
    }

  return 1;
}

int buffer_pool_alloc(struct pool *p, struct buffer ** tbuf)
{
  int i;

  for (i=0;i<p->buf_nb;i++)
    if (!p->buf_list[i].allocated)
      {
	p->buf_list[i].allocated = 1;
	*tbuf = &p->buf_list[i];
	/*
	{
	  struct timeval tv;
	  gettimeofday(&tv,NULL);
	  printf("%d.%06d: allocation pool %p, tbuf %p, buf %p\n",
		 tv.tv_sec,tv.tv_usec,
		 p,*tbuf,(*tbuf)->buf);
	}
	*/
	return 1;
      }

  return 0;
}

int buffer_pool_free(struct pool *p, struct buffer * tbuf)
{
  if (!tbuf->allocated)
    return 0;
  /*
  {
    struct timeval tv;
    gettimeofday(&tv,NULL);
    printf("%d.%06d: free       pool %p, tbuf %p, buf %p\n",
	   tv.tv_sec,tv.tv_usec,
	   p,tbuf,tbuf->buf);
  }
  */
  tbuf->allocated = 0;
  return 1;
}

struct buffer * buffer_pool_search(struct pool *p, void *ptr)
{
  int i;

  for (i=0;i<p->buf_nb;i++)
    if (p->buf_list[i].buf == ptr)
      return &p->buf_list[i];

  return NULL;
}
